@@ -70,6 +70,9 @@ typedef enum HdffMessageId_t
     HDFF_MSG_AV_SET_AUDIO_DOWNMIX,
     HDFF_MSG_AV_SET_AUDIO_CHANNEL,
     HDFF_MSG_AV_SET_PLAY_MODE,
+    HDFF_MSG_AV_SET_OPTIONS,
+    HDFF_MSG_AV_MUTE_AUDIO,
+    HDFF_MSG_AV_MUTE_VIDEO,
 
     HDFF_MSG_MUX_SET_VIDEO_OUT = 0,
     HDFF_MSG_MUX_SET_SLOW_BLANK,
@@ -103,6 +106,7 @@ typedef enum HdffMessageId_t
     HDFF_MSG_OSD_DRAW_TEXT,
     HDFF_MSG_OSD_DRAW_WIDE_TEXT,
     HDFF_MSG_OSD_DRAW_BITMAP,
+    HDFF_MSG_OSD_DRAW_UTF8_TEXT,
 
     HDFF_MSG_HDMI_ENABLE_OUTPUT = 0,
     HDFF_MSG_HDMI_SET_VIDEO_MODE,
